@@ -9,8 +9,11 @@ use Carp qw(croak);
 use Archive::Tar;
 use File::Spec::Functions qw(file_name_is_absolute rel2abs);
 use File::Basename qw(dirname fileparse);
+use MIME::Base64 qw(decode_base64);
+use IO::Uncompress::Gunzip;
+use HTTP::Tiny;
 
-our $VERSION = "0.7";
+our $VERSION = "0.8";
 
 =pod
 
@@ -34,6 +37,14 @@ or
   use JSON::PP;
   use YAML::PP;
 
+or
+
+  use lib::archive '__DATA__';
+
+  __DATA__
+  <followed by base64 encoded tar or tgz blocks separated by an empty line>
+
+
 =head1 DESCRIPTION
 
 Specify TAR archives to directly load modules from. The TAR files will be
@@ -46,8 +57,8 @@ calling script or module resides in. So don't do a chdir() before using
 lib::archive when you call your script with a relative path B<and> use releative
 paths for lib::archive.
 
-B<The module will not create any files, not even temporary. Everything is
-extracted on the fly>.
+B<The module will not create any files, not even temporary.
+Everything is extracted on the fly>.
 
 You can use every file format Archive::Tar supports.
 
@@ -96,18 +107,22 @@ Thomas Kratz E<lt>tomk@cpan.orgE<gt>
 =cut
 
 my $cpan   = $ENV{CPAN_MIRROR} || 'https://www.cpan.org';
-my $is_url = qr!^(?:CPAN|https?)://!;
+my $rx_url = qr!^(?:CPAN|https?)://!;
 my $tar    = Archive::Tar->new();
 
+
 sub import {
-    my $class = shift;
+    my ( $class, @entries ) = @_;
     my %cache;
 
-    ( my $acdir = dirname( rel2abs( (caller)[1] ) ) ) =~ s!\\!/!g;
+    my $caller_file = (caller)[1];
 
-    for my $entry (@_) {
-        my $is_url = $entry =~ /$is_url/;
-        my $arcs = $is_url ? _get_url($entry) : _get_files( $entry, $acdir );
+    for my $entry (@entries) {
+        my $is_url = $entry =~ /$rx_url/;
+        my $arcs
+            = $is_url                  ? _get_url($entry)
+            : ( $entry eq '__DATA__' ) ? _get_data($caller_file)
+            :                            _get_files( $entry, $caller_file );
         for my $arc (@$arcs) {
             my $path = $is_url ? $entry : $arc->[0];
             my %tmp;
@@ -127,19 +142,23 @@ sub import {
             }
         }
     }
+
     unshift @INC, sub {
         my ( $cref, $rel ) = @_;
         return unless my $rec = $cache{$rel};
-        $INC{$rel} = $rec->{path};
-        open(my $fh, '<', $rec->{content});
-        return $fh;
+        $INC{$rel} = $rec->{path} unless defined($DB::single);    ## no critic (RequireLocalizedPunctuationVars)
+        open( my $pfh, '<', $rec->{content} ) or croak $!;        ## no critic (RequireBriefOpen)
+        return $pfh;
     };
+
+    return;
 }
 
 
 sub _get_files {
-    my ( $glob, $cdir ) = @_;
-    ( my $glob_ux = $glob ) =~ s!\\!/!g;
+    my ( $glob, $cfile ) = @_;
+    ( my $glob_ux = $glob )                      =~ s!\\!/!g;
+    ( my $cdir    = dirname( rel2abs($cfile) ) ) =~ s!\\!/!g;
     $glob_ux = "$cdir/$glob_ux" unless file_name_is_absolute($glob_ux);
     my @files;
     for my $f ( sort glob($glob_ux) ) {
@@ -154,13 +173,10 @@ sub _get_url {
     my ($url) = @_;
 
     my ($module) = $url =~ m!/([^/]+)\.tar\.gz$!;
-    my ($top) = split( /-/, $module );
+    my ($top)    = split( /-/, $module );
 
     $url =~ s!^CPAN://!$cpan/modules/by-module/$top/!;
 
-    require HTTP::Tiny;
-    require IO::Uncompress::Gunzip;
-
     my $rp = HTTP::Tiny->new->get($url);
 
     my @zips;
@@ -175,4 +191,28 @@ sub _get_url {
 }
 
 
+sub _get_data {
+    my ($cfn) = @_;
+    open( my $fh, '<', $cfn ) or croak "couldn't open $cfn, $!";
+    local $/ = undef;
+    my $data = <$fh>;
+    close($fh);
+    $data =~ s/^.*\n__DATA__\r?\n/\n/s;
+    my @data = split( /\n\n+/, $data );
+    my @tars;
+
+    for my $d (@data) {
+        my $content = decode_base64($d);
+        my $z = eval { IO::Uncompress::Gunzip->new( \$content ) };
+        if ($z) {
+            push @tars, [ $z, '' ];
+            next;
+        }
+        open( my $cfh, '<', \$content ) or croak $!;    ## no critic (RequireBriefOpen)
+        push @tars, [ $cfh, '' ];
+    }
+    return \@tars;
+}
+
+
 1;
